// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// combase.h
//
// Class common to all objects, contains shared data.  Currently,
// this class is used to store the filename and line number for error
// reporting.  Thus any object that wish to report and error with
// filename and line number, it has to inheret from this common_base
// class.

// RegDecl will inherit from both TFDecl and ModuleItem.  But we need
// <filename> for both.  Since the filename will be the same, we can
// try use a virtual base class.
//
// 95-9-16.  Since all classes have these informations, it was too
// wasteful to not to put in it a common object.

#ifndef _COMBASE_H
#define _COMBASE_H

#include <iostream.h>
#include <string>

struct common_base
	{
	// Basic types used by many objects.
	typedef string str_type;
	typedef ostream ostream_type;

	common_base(str_type fn = str_type("unknown"), long ln = -1)
		: _filename(fn), _lineno(ln) {}
	common_base(const common_base &cm)
		: _filename(cm._filename), _lineno(cm._lineno) {}
	virtual ~common_base()
		{}

	str_type _filename; // Data for error reporting.
	long _lineno;
	};

#endif // _COMBASE_H
