// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// hash.h
//
// See "Compiler Design in C" by Allen Holub, (C) 1990 Prentice Hall.
// The hash value object is used by the symbol table to lookup and
// store symbols (names) into the table.  In the future, I hope to use
// one of the ANSI C++ STL containers to do this.  But even these
// containers need a key.  So we still need a hash object.

#ifndef _HASH_H
#define _HASH_H

#include <string>

class ostream;

// If the size of the symbol table is not known at the time the hash
// function was called.  Usually means the symbol table was not
// initialized yet.  This error value is in sync with the error
// reporting mechanism used by VBS.  
#define INIT_ERROR_VALUE 1

struct hash_value
	{
	typedef unsigned long index_type;
	typedef signed scope_type;
	typedef string str_type;

	hash_value(index_type idx = 0, scope_type sc = -1)
		: _value(idx), _scope(sc) {}
	hash_value(const hash_value &hv)
		: _value(hv._value), _scope(hv._scope) {}
	hash_value &operator=(const hash_value &hv)
		{ _value = hv._value; _scope = hv._scope; return *this; }

	index_type _value;
	scope_type _scope;
	};

hash_value hash(const hash_value::str_type &, hash_value::scope_type);
void rehash(hash_value &, hash_value::index_type);
ostream &operator<<(ostream &, const hash_value &);

extern unsigned long TABLESIZE;

#endif // _HASH_H
