// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_func.cc

#include "common/st_func.h"

st_function::st_function(const str_type &n, stmt_type *s, decl_list *d)
	: _done_setup(false), _name(n)
	{
	_stmt = s;
	_iovariable = 0;
	_declaration = d;
	_monitor = 0;
	}

st_function::~st_function()
	{
	delete _stmt;
	delete _iovariable;
	delete _monitor;
	}

const st_function::str_type
st_function::type_name() const
	{ return "function"; }

const st_function::str_type
st_function::name() const
	{ return _name; }

const st_function::scope_type
st_function::scope() const
	{ return _scope; }

void
st_function::scope(scope_type sc)
	{ _scope = sc; }

st_function *
st_function::get_function()
	{ return this; }

void
st_function::assignment(num_type &bv , position_type ms, position_type ls)
	{
	if (ms >= 0 && ls >= 0)
		_storage(ms, ls) = bv;
	else
		_storage = bv;
	}

st_function::ostream_type &
st_function::display(ostream_type &s) const
	{
	s << "('" << _name << "'," << type_name() << ',' << _scope << ')';
	return s;
	}

bool
st_function::sim_monitor(const monitor_type &mon)
	{ return mon(this); }

st_function::num_type
st_function::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
st_function::sim_setup(const setup_type &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const st_function &d)
	{ return d.display(s); }
