// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_func.h
//
// Symbol table node to store a function definition.
//
// Note:  Function definitions are setup only if it is enabled in the
// module it was defined in. Thus, the tf declarations must be saved
// until setup is done.

#ifndef _ST_FUNC_H
#define _ST_FUNC_H

#include "common/ptr.h"
#include "common/event.h"
#include "stmt/stmtbase.h"
#include "moditm/tfbase.h"
#include "common/st_node.h"

class st_function : public st_node_base
	{
	friend struct monitor_st_node;
	friend struct evaluate_st_node;
	friend struct setup_st_node;

	friend struct setup_expr;
	friend struct setup_stmt;
	friend struct trigger_stmt;
	friend struct setup_module_item;

public:
	typedef st_node_base::num_type num_type;
	typedef st_node_base::scope_type scope_type;

	typedef num_type::position_type position_type;
	typedef stmt_base stmt_type;
	typedef tfdecl_base decl_type;
	typedef basic_ptr<decl_type> decl_ptr;
	typedef list<decl_ptr> decl_list;
	typedef list<hash_type> io_list;
	typedef event_base<stmt_type> event_type;
	typedef counted_ptr<event_type> event_ptr;
	typedef list<event_ptr> monitor_list;

	st_function(const str_type &, stmt_type *, decl_list * = 0);
	~st_function();

	const str_type type_name() const;
	const str_type name() const;
	const scope_type scope() const;
	void scope(scope_type);
	st_function *get_function();

	ostream_type &display(ostream_type &) const;
	bool sim_monitor(const monitor_type &);
	num_type sim_evaluate(const evaluate_type &) const;
	void sim_setup(const setup_type &);

	// Utility for function trigger.
	void assignment(num_type &, position_type = -1, position_type = -1);

private:
	bool _done_setup;
	scope_type _scope;
	str_type _name;
	num_type _storage;
	stmt_type *_stmt;
	io_list *_iovariable;
	decl_list *_declaration;

	// List of monitor statements for functions.
	monitor_list *_monitor;
	};

ostream &operator<<(ostream &, const st_function &);

#endif // _ST_FUNC_H
