// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_module.cc

#include "misc/port.h"
#include "moditm/mibase.h"
#include "common/st_mod.h"

st_module::st_module(const str_type &n)
	: _scope(-1), _name(n)
	{
	_iovariable = 0;
	_port_list = 0;
	_module_item = 0;
	}

st_module::~st_module()
	{
	delete _iovariable;
	delete _port_list;
	delete _module_item;
	}

const st_module::str_type
st_module::type_name() const
	{ return "module"; }

const st_module::str_type
st_module::name() const
	{ return _name; }

const st_module::scope_type
st_module::scope() const
	{ return _scope; }

void
st_module::scope(scope_type sc)
	{ _scope = sc; }

st_module *
st_module::get_module()
	{ return this; }

st_module::ostream_type &
st_module::display(ostream_type &s) const
	{
	s << "('" << _name << "'," << type_name() << ',' << _scope << ')';
	return s;
	}

void
st_module::sim_setup(const setup_type &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const st_module &d)
	{ return d.display(s); }
