// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_module.h
//
// Symbol table node to store module definition.

#ifndef _ST_MODULE_H
#define _ST_MODULE_H

#include <list>
#include "common/ptr.h"
#include "common/st_node.h"
#include "misc/port.h"
#include "moditm/mibase.h"

class st_module : public st_node_base
	{
	friend struct setup_st_node;
	friend struct setup_module;

public:
	typedef list<hash_type> io_list;
	typedef port port_type;
	typedef basic_ptr<port_type> port_ptr;
	typedef list<port_ptr> port_list;
	typedef module_item_base module_item_type;
	typedef basic_ptr<module_item_type> module_item_ptr;
	typedef list<module_item_ptr> module_item_list;

	st_module(const str_type &);
	~st_module();

	const str_type type_name() const;
	const str_type name() const;
	const scope_type scope() const;
	void scope(scope_type);
	st_module *get_module();

	ostream_type &display(ostream_type &s) const;
	void sim_setup(const setup_type &);

private:
	scope_type _scope;
	str_type _name;
	io_list *_iovariable;
	port_list *_port_list;
	module_item_list *_module_item;
	};

ostream &operator<<(ostream &, const st_module &);

#endif // _ST_MODULE_H
