// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_net.cc

#include "common/st_net.h"

st_net::st_net(const str_type &n, net_type nt, iodirection_type iodir)
	: _type(nt), _iodirection(iodir), _scope(-1), _name(n),
		_port_ms(-1), _port_ls(-1)
	{
	_storage = 0;
	_port_index = 0;
	_propagate = 0;
	_monitor = 0;
	}

st_net::~st_net()
	{
	delete _storage;
	delete _port_index;
	delete _propagate;
	delete _monitor;
	}

const st_net::str_type
st_net::type_name() const
	{
	if (_type == WIRE)
		return "wire";
	else if (_type == REGISTER)
		return "reg";
	return "port";
	}

const st_net::str_type
st_net::name() const
	{ return _name; }

const st_net::scope_type
st_net::scope() const
	{ return _scope; }

void
st_net::scope(scope_type sc)
	{ _scope = sc; }

bool
st_net::complete() const
	{
	st_net_complete complete(*this);
	return complete();
	}

st_net *
st_net::get_net()
	{ return this; }

st_net::ostream_type &
st_net::display(ostream_type &s) const
	{
	s << "('" << _name << "'," << type_name() << ',' << _scope << ')';
	return s;
	}

bool
st_net::sim_monitor(const monitor_type &mon)
	{ return mon(this); }

st_net::num_type
st_net::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
st_net::sim_setup(const setup_type &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const st_net &d)
	{ return d.display(s); }


// Trigger member utility functions.


void
st_net::assignment(num_type &num, position_type m, position_type l)
	{
	st_net_assign assign(*this);
	assign(num, m, l);
	}
