// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// st_task.h
//
// Symbol table node to store a task definition.
//
// Note:  Task definitions are setup only if it is enabled in the
// module it was defined in. Thus, the tf declarations must be saved
// until setup is done.

#ifndef _ST_TASK_H
#define _ST_TASK_H

#include <list>
#include "common/ptr.h"
#include "stmt/stmtbase.h"
#include "moditm/tfbase.h"
#include "common/st_node.h"

class st_task : public st_node_base
	{
	friend struct setup_st_node;
	friend struct trigger_st_node;

	friend struct setup_stmt;
	friend struct setup_module_item;

public:
	typedef stmt_base stmt_type;
	typedef list<hash_type> io_list;
	typedef tfdecl_base decl_type;
	typedef basic_ptr<decl_type> decl_ptr;
	typedef list<decl_ptr> decl_list;

	st_task(const str_type &, stmt_type *, decl_list * = 0);
	~st_task();

	const str_type type_name() const;
	const str_type name() const;
	const scope_type scope() const;
	void scope(scope_type);
	st_task *get_task();

	ostream_type &display(ostream_type &s) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	bool _done_setup;
	scope_type _scope;
	str_type _name;
	stmt_type *_stmt;
	io_list *_iovariable;
	decl_list *_declaration;
	};

ostream &operator<<(ostream &, const st_task &);

#endif // _ST_TASK_H
