// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// testtw.cc

#include <string>
#include "time_whl.h"

class Obj
	{
public:
	Obj(char *);
	ostream &display(ostream &) const;
	bool sim_trigger(Obj *);

private:
	string str;
	};

template<class T>
class twhandler : public time_wheel_handler<T>
	{
	void operator()(time_type t, list<T *> &lst) const
		{
		list<T *>::iterator itp(lst.begin());
		list<T *>::iterator stop(lst.end());
		cout << "Time " << t << ':' << endl;
		for (; itp != stop; ++itp)
			(*itp)->sim_trigger(*itp);
		}
	};

time_wheel<Obj> timewheel;

int
main()
	{
	Obj a("abc"), b("bcd"), c("cde");

	timewheel.sim_add_event(0, &a);
	timewheel.sim_add_event(15, &b);
	timewheel.sim_add_event(20, &c);
	timewheel.display(cout);
	twhandler<Obj> x;
	timewheel.sim_trigger(x);
	cout << "We're done" << endl;

	return 0;
	}


Obj::Obj(char *s)
	: str(s) {}

ostream &
Obj::display(ostream &s) const
	{ s << str << endl; return s; }

bool
Obj::sim_trigger(Obj *)
	{
	cout << "At " << timewheel.current_time()
		<< " triggering: " << str << endl;
	return true;
	}
