// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// add.cc

#include "expr/add.h"

add::add(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

add::add(const add &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

add::~add()
	{ delete _left; delete _right; }

add *
add::copy_constructor() const
	{ return new add(*this); }

add::ostream_type &
add::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "+"); }

bool
add::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

add::num_type
add::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
add::sim_setup(const setup_type &setup)
	{ setup(this); }

void
add::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const add &d)
	{ return d.display(s); }
