// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// eeval.cc

#include "common/sym_tab.h"
#include "common/st_net.h"
#include "common/st_func.h"
#include "common/st_eval.h"
#include "expr/eeval.h"
#include "expr/add.h"
#include "expr/andand.h"
#include "expr/eqeq.h"
#include "expr/funcenbl.h"
#include "expr/grteq.h"
#include "expr/grtthan.h"
#include "expr/invert.h"
#include "expr/leseq.h"
#include "expr/lesthan.h"
#include "expr/logand.h"
#include "expr/logor.h"
#include "expr/logxor.h"
#include "expr/lshift.h"
#include "expr/not.h"
#include "expr/noteq.h"
#include "expr/number.h"
#include "expr/oror.h"
#include "expr/rangeid.h"
#include "expr/rshift.h"
#include "expr/sub.h"

extern symbol_table symboltable;

evaluate_expr::num_type
evaluate_expr::operator()(const add *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		+  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const and_and *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		&& p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const equal_equal *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		== p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const func_enable *p) const
	{
	st_function *func = symboltable.get(p->_index)->get_function();
	return func->sim_evaluate(evaluate_st_node(p));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const grt_equal *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		>= p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const grt_than *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		>  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const invert *p) const
	{
	return ~(p->_expr->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const left_shift *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		<< (unsigned long) p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const les_equal *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		<= p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const les_than *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		<  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const logic_and *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		&  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const logic_or *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		|  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const logic_xor *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		^  p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const not_expr *p) const
	{
	return !(p->_expr->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const not_equal *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		!= p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const number *p) const
	{ return *p; }

evaluate_expr::num_type
evaluate_expr::operator()(const or_or *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		|| p->_right->sim_evaluate(evaluate_expr()));
	}

// qouted_str uses default evaluation, return DC.

evaluate_expr::num_type
evaluate_expr::operator()(const range_id *p) const
	{
	st_net *node = symboltable.get(p->_index)->get_net();
	evaluate_expr::num_type res(node->sim_evaluate(evaluate_st_node()));
	// Setup should have checked range.
	range_id::position_type ms, ls;
	range_id::select_size_type type = p->get_select(ms, ls);
	if (type == range_id::select_type::SELECT_RANGE)
		return res(ms, ls);
	else if (type == range_id::select_type::SELECT_BIT)
		return res(ms);
	return res;
	}

evaluate_expr::num_type
evaluate_expr::operator()(const right_shift *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		>> (unsigned long) p->_right->sim_evaluate(evaluate_expr()));
	}

evaluate_expr::num_type
evaluate_expr::operator()(const sub *p) const
	{
	return (p->_left->sim_evaluate(evaluate_expr())
		-  p->_right->sim_evaluate(evaluate_expr()));
	}
