// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// emon.cc

#include "common/sym_tab.h"
#include "common/st_net.h"
#include "common/st_func.h"
#include "common/st_mon.h"
#include "expr/add.h"
#include "expr/andand.h"
#include "expr/eqeq.h"
#include "expr/funcenbl.h"
#include "expr/grteq.h"
#include "expr/grtthan.h"
#include "expr/invert.h"
#include "expr/leseq.h"
#include "expr/lesthan.h"
#include "expr/logand.h"
#include "expr/logor.h"
#include "expr/logxor.h"
#include "expr/lshift.h"
#include "expr/not.h"
#include "expr/noteq.h"
#include "expr/number.h"
#include "expr/oror.h"
#include "expr/qstr.h"
#include "expr/rangeid.h"
#include "expr/rshift.h"
#include "expr/sub.h"
#include "expr/eeval.h"
#include "expr/emon.h"

extern symbol_table symboltable;

monitor_expr::monitor_expr(event_ptr *e)
	{ _event = e; }

bool
monitor_expr::operator()(const add *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const and_and *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const equal_equal *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const func_enable *p) const
	{
	st_function *func = symboltable.get(p->_index)->get_function();
	if (func == 0)
		return false;
	return func->sim_monitor(monitor_st_node(_event));
	}

bool
monitor_expr::operator()(const grt_equal *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const grt_than *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const invert *p) const
	{ return p->_expr->sim_monitor(monitor_expr(_event)); }

bool
monitor_expr::operator()(const left_shift *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const les_equal *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const les_than *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const logic_and *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const logic_or *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const logic_xor *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const not_expr *p) const
	{ return p->_expr->sim_monitor(monitor_expr(_event)); }

bool
monitor_expr::operator()(const not_equal *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const number *) const
	{ return true; }

bool
monitor_expr::operator()(const or_or *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const qouted_str *) const
	{ return true; }

// qouted_str uses default evaluation, return DC.

bool
monitor_expr::operator()(const range_id *p) const
	{
	st_net *node = symboltable.get(p->_index)->get_net();
	if (node == 0)
		return false;
	return node->sim_monitor(monitor_st_node(_event));
	}

bool
monitor_expr::operator()(const right_shift *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}

bool
monitor_expr::operator()(const sub *p) const
	{
	return (p->_left->sim_monitor(monitor_expr(_event))
	     && p->_right->sim_monitor(monitor_expr(_event)));
	}
