// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// eqeq.cc

#include "expr/eqeq.h"

equal_equal::equal_equal(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

equal_equal::equal_equal(const equal_equal &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

equal_equal::~equal_equal()
	{ delete _left; delete _right; }

equal_equal *
equal_equal::copy_constructor() const
	{ return new equal_equal(*this); }

equal_equal::ostream_type &
equal_equal::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "=="); }

bool
equal_equal::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

equal_equal::num_type
equal_equal::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
equal_equal::sim_setup(const setup_type &setup)
	{ setup(this); }

void
equal_equal::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const equal_equal &d)
	{ return d.display(s); }
