// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// etrigger.cc

#include "common/sym_tab.h"
#include "common/st_net.h"
#include "common/st_func.h"
#include "common/st_eval.h"
#include "expr/eeval.h"
#include "expr/etrigger.h"
#include "expr/add.h"
#include "expr/andand.h"
#include "expr/eqeq.h"
#include "expr/funcenbl.h"
#include "expr/grteq.h"
#include "expr/grtthan.h"
#include "expr/invert.h"
#include "expr/lshift.h"
#include "expr/leseq.h"
#include "expr/lesthan.h"
#include "expr/logand.h"
#include "expr/logor.h"
#include "expr/logxor.h"
#include "expr/not.h"
#include "expr/noteq.h"
#include "expr/number.h"
#include "expr/oror.h"
#include "expr/qstr.h"
#include "expr/rangeid.h"
#include "expr/rshift.h"
#include "expr/sub.h"

extern symbol_table symboltable;

void
display_output(const trigger_expr::str_type &s, const trigger_expr::num_type &n)
	{
	trigger_expr::num_type::base_type base = trigger_expr::num_type::BASE10;
	trigger_expr::num_type::size_type size = 0;
	get_base_size(s, base, size);
	cout << n.to_string(base, size);
	}

trigger_expr::trigger_expr(str_type &s)
	: _format(s) {}

void
trigger_expr::operator()(add *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(and_and *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(equal_equal *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(func_enable *p) const
	{
	// <node> can not be NULL.  Setup should have checked.
	st_function *node = symboltable.get(p->_index)->get_function();
	display_output(_format, node->sim_evaluate(evaluate_st_node(p)));
	}

void
trigger_expr::operator()(grt_equal *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(grt_than *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(invert *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(left_shift *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(les_equal *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(les_than *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(logic_and *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(logic_or *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(logic_xor *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(not_expr *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(not_equal *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(number *n) const
	{ display_output(_format, *n); }

void
trigger_expr::operator()(or_or *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(qouted_str *p) const
	{
	// Display the string without the qoutes.
	if (_format != "s" && _format != "S")
		cout << 'x';
	else
		{
		size_t len = p->size() - 1;
		const str_type &s = *p;
		for (size_t i = 1; i < len; ++i)
			cout << s[i];
		}
	}

void
trigger_expr::operator()(range_id *p) const
	{
	// Grab the data from the symbol table.
	st_net *node = symboltable.get(p->_index)->get_net();
	num_type res(node->sim_evaluate(evaluate_st_node()));
	// Print out the content.
	range_id::position_type ms, ls;
	range_id::select_size_type type = p->get_select(ms, ls);
	// Setup should have checked range.
	if (type == range_id::select_type::SELECT_RANGE)
		display_output(_format, res(ms, ls));
	else if (type == range_id::select_type::SELECT_BIT)
		display_output(_format, res(ms));
	else
		display_output(_format, res);
	}

void
trigger_expr::operator()(right_shift *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }

void
trigger_expr::operator()(sub *p) const
	{ display_output(_format, p->sim_evaluate(evaluate_expr())); }
