// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// funcenbl.cc

#include "expr/funcenbl.h"

func_enable::func_enable(const char *n, arg_list *a)
	: _index(), _name(n) { _argument = a; }

func_enable::func_enable(const func_enable &p)
	: common_base(p), expr_base(p), _index(), _name(p._name)
	{
	if (p._argument != 0)
		{
		_argument = new arg_list;
		arg_list::iterator itp(p._argument->begin());
		arg_list::iterator stop(p._argument->end());
		for (; itp != stop; ++itp)
			_argument->push_back((*itp)->copy_constructor());
		}
	else
		_argument = 0;
	}

func_enable::~func_enable()
	{ delete _argument; }

func_enable::hash_type
func_enable::index() const
	{ return _index; }

func_enable *
func_enable::get_func_enable()
	{ return this; }

func_enable *
func_enable::copy_constructor() const
	{ return new func_enable(*this); }

func_enable::ostream_type &
func_enable::display(ostream_type &s) const
	{
	s << _name << '(';
	if (_argument)
		{
		arg_list::iterator itp(_argument->begin());
		arg_list::iterator stop(_argument->end());
		(*itp)->display(s);
		for (++itp; itp != stop; ++itp)
			{
			s << ", ";
			(*itp)->display(s);
			}
		}
	s << ")";
	return s;
	}

bool
func_enable::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

func_enable::num_type
func_enable::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
func_enable::sim_setup(const setup_type &setup)
	{ setup(this); }

void
func_enable::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const func_enable &d)
	{ return d.display(s); }
