// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// funcenbl.h

#ifndef _FUNCENBL_H
#define _FUNCENBL_H

#include <list>
#include "common/ptr.h"
#include "common/hash.h"
#include "expr/exprbase.h"

class func_enable : public expr_base
	{
	friend struct monitor_expr;
	friend struct evaluate_expr;
	friend struct setup_expr;
	friend struct trigger_expr;

	friend struct evaluate_st_node;

public:
	typedef hash_value hash_type;
	typedef expr_base expr_type;
	typedef basic_ptr<expr_type> arg_type;
	typedef list<arg_type> arg_list;

	func_enable(const char *, arg_list * = 0);
	func_enable(const func_enable &);
	~func_enable();

	const str_type name() const;
	hash_type index() const;
	arg_list *argument_list() const;
	func_enable *get_func_enable();

	func_enable *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	bool sim_monitor(const monitor_type &) const;
	num_type sim_evaluate(const evaluate_type &) const;
	void sim_setup(const setup_type &);
	void sim_trigger(const trigger_type &);

private:
	hash_type _index; // Index of function into symbol table.
	str_type _name;
	arg_list *_argument;
	};

ostream &operator<<(ostream &, const func_enable &);

#endif // _FUNCENBL_H
