// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// leseq.cc

#include "expr/leseq.h"

les_equal::les_equal(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

les_equal::les_equal(const les_equal &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

les_equal::~les_equal()
	{ delete _left; delete _right; }

les_equal *
les_equal::copy_constructor() const
	{ return new les_equal(*this); }

les_equal::ostream_type &
les_equal::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "<="); }

bool
les_equal::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

les_equal::num_type
les_equal::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
les_equal::sim_setup(const setup_type &setup)
	{ setup(this); }

void
les_equal::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const les_equal &d)
	{ return d.display(s); }
