// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// logand.cc

#include "expr/logand.h"

logic_and::logic_and(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

logic_and::logic_and(const logic_and &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

logic_and::~logic_and()
	{ delete _left; delete _right; }

logic_and *
logic_and::copy_constructor() const
	{ return new logic_and(*this); }

logic_and::ostream_type &
logic_and::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "&"); }

bool
logic_and::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

logic_and::num_type
logic_and::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
logic_and::sim_setup(const setup_type &setup)
	{ setup(this); }

void
logic_and::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const logic_and &d)
	{ return d.display(s); }
