// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// logor.cc

#include "expr/logor.h"

logic_or::logic_or(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

logic_or::logic_or(const logic_or &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

logic_or::~logic_or()
	{ delete _left; delete _right; }

logic_or *
logic_or::copy_constructor() const
	{ return new logic_or(*this); }

logic_or::ostream_type &
logic_or::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "|"); }

bool
logic_or::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

logic_or::num_type
logic_or::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
logic_or::sim_setup(const setup_type &setup)
	{ setup(this); }

void
logic_or::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const logic_or &d)
	{ return d.display(s); }
