// Verilog Behavioral Simulator
// Copyright (C) 1995 Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// logxor.cc

#include "expr/logxor.h"

logic_xor::logic_xor(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

logic_xor::logic_xor(const logic_xor &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

logic_xor::~logic_xor()
	{ delete _left; delete _right; }

logic_xor *
logic_xor::copy_constructor() const
	{ return new logic_xor(*this); }

logic_xor::ostream_type &
logic_xor::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "^"); }

bool
logic_xor::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

logic_xor::num_type
logic_xor::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
logic_xor::sim_setup(const setup_type &setup)
	{ setup(this); }

void
logic_xor::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const logic_xor &d)
	{ return d.display(s); }
