// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// lshift.cc

#include "expr/lshift.h"

left_shift::left_shift(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

left_shift::left_shift(const left_shift &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

left_shift::~left_shift()
	{ delete _left; delete _right; }

left_shift *
left_shift::copy_constructor() const
	{ return new left_shift(*this); }

left_shift::ostream_type &
left_shift::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "<<"); }

bool
left_shift::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

left_shift::num_type
left_shift::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
left_shift::sim_setup(const setup_type &setup)
	{ setup(this); }

void
left_shift::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const left_shift &d)
	{ return d.display(s); }
