// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// noteq.cc

#include "expr/noteq.h"

not_equal::not_equal(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

not_equal::not_equal(const not_equal &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

not_equal::~not_equal()
	{ delete _left; delete _right; }

not_equal *
not_equal::copy_constructor() const
	{ return new not_equal(*this); }

not_equal::ostream_type &
not_equal::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "!="); }

bool
not_equal::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

not_equal::num_type
not_equal::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
not_equal::sim_setup(const setup_type &setup)
	{ setup(this); }

void
not_equal::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const not_equal &d)
	{ return d.display(s); }
