// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// oror.cc

#include "expr/oror.h"

or_or::or_or(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

or_or::or_or(const or_or &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

or_or::~or_or()
	{ delete _left; delete _right; }

or_or *
or_or::copy_constructor() const
	{ return new or_or(*this); }

or_or::ostream_type &
or_or::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "||"); }

bool
or_or::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

or_or::num_type
or_or::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
or_or::sim_setup(const setup_type &setup)
	{ setup(this); }

void
or_or::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const or_or &d)
	{ return d.display(s); }
