// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// rangeid.cc

#include "expr/rangeid.h"

range_id::range_id(const char *n, select_type *r)
	: _index(), _name(n)
	{ _select = r; }

range_id::range_id(const range_id &p)
	: common_base(p), expr_base(p), _index(), _name(p._name)
	{
	if (p._select != 0)
		_select = p._select->copy_constructor();
	else
		_select = 0;
	}

range_id::~range_id()
	{ delete _select; }

const range_id::str_type
range_id::name() const
	{ return _name; }

range_id::hash_type
range_id::index() const
	{ return _index; }

void
range_id::index(const hash_type &h)
	{ _index = h; }

range_id::select_size_type
range_id::get_select(position_type &ms, position_type &ls) const
	{
	select_size_type type = select_type::SELECT_NONE;
	if (_select != 0)
		type = _select->get_data(ms, ls);
	else
		{
		ms = -1;
		ls = -1;
		}
	return type;
	}

range_id *
range_id::get_range_id()
	{ return this; }

range_id *
range_id::copy_constructor() const
	{ return new range_id(*this); }

range_id::ostream_type &
range_id::display(ostream_type &s) const
	{
	s << _name;
	if (_select != 0)
		s << *_select;
	return s;
	}

bool
range_id::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

range_id::num_type
range_id::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
range_id::sim_setup(const setup_type &setup)
	{ setup(this); }

void
range_id::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const range_id &d)
	{ return d.display(s); }
