// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// rangeid.h

#ifndef _RANGEID_H
#define _RANGEID_H

#include "common/hash.h"
#include "common/combase.h"
#include "expr/exprbase.h"
#include "misc/selbase.h"

class range_id : public expr_base
	{
	friend struct monitor_expr;
	friend struct evaluate_expr;
	friend struct setup_expr;
	friend struct trigger_expr;

	friend struct setup_lvalue;
	friend struct trigger_lvalue;

public:
	typedef hash_value hash_type;
	typedef select_base select_type;
	typedef select_type::SELECT_SIZE select_size_type;
	typedef select_type::position_type position_type;

	range_id(const char *, select_type *r);
	range_id(const range_id &);
	~range_id();

	const str_type name() const;
	hash_type index() const;
	void index(const hash_type &);
	select_size_type get_select(position_type &, position_type &) const;
	range_id *get_range_id();

	range_id *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	bool sim_monitor(const monitor_type &) const;
	num_type sim_evaluate(const evaluate_type &) const;
	void sim_setup(const setup_type &);
	void sim_trigger(const trigger_type &);

protected:
	hash_type _index; // Index into symbol table.
	str_type _name; // Name of variable.
	select_type *_select; // Optional range/bit select.
	};

ostream &operator<<(ostream &, const range_id &);

#endif // _RANGEID_H
