// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// rshift.cc

#include "expr/rshift.h"

right_shift::right_shift(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

right_shift::right_shift(const right_shift &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

right_shift::~right_shift()
	{ delete _left; delete _right; }

right_shift *
right_shift::copy_constructor() const
	{ return new right_shift(*this); }

right_shift::ostream_type &
right_shift::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, ">>"); }

bool
right_shift::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

right_shift::num_type
right_shift::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
right_shift::sim_setup(const setup_type &setup)
	{ setup(this); }

void
right_shift::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const right_shift &d)
	{ return d.display(s); }
