// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// sub.h

#include "expr/sub.h"

sub::sub(expr_type *l, expr_type *r)
	{ _left = l; _right = r; }

sub::sub(const sub &p)
	: common_base(p), expr_base(p)
	{
	_left = p._left->copy_constructor();
	_right = p._right->copy_constructor();
	}

sub::~sub()
	{ delete _left; delete _right; }

sub *
sub::copy_constructor() const
	{ return new sub(*this); }

sub::ostream_type &
sub::display(ostream_type &s) const
	{ return display_binary_expr(s, *_left, *_right, "-"); }

bool
sub::sim_monitor(const monitor_type &mon) const
	{ return mon(this); }

sub::num_type
sub::sim_evaluate(const evaluate_type &eval) const
	{ return eval(this); }

void
sub::sim_setup(const setup_type &setup)
	{ setup(this); }

void
sub::sim_trigger(const trigger_type &trigger)
	{ trigger(this); }

ostream &
operator<<(ostream &s, const sub &d)
	{ return d.display(s); }
