// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// decbase.h
//
// Base class for all delay event controls.

#ifndef _DECBASE_H
#define _DECBASE_H

#include "common/combase.h"

struct dec_setup;
struct dec_trigger;
class delay_num;
class delay_id;
class ored_event_expr;

struct dec_base : public virtual common_base
	{
	typedef dec_setup setup_type;
	typedef dec_trigger trigger_type;

	dec_base()
		: _persist(false), _delayed_loop(false)
		{}
	dec_base(const dec_base &p)
		: common_base(p), _persist(p._persist),
			_delayed_loop(p._delayed_loop)
		{}

	virtual dec_base *copy_constructor() const
		{ return new dec_base(*this); }
	virtual ostream_type &display(ostream_type &s) const
		{ return s; }
	virtual void sim_setup(const setup_type &)
		{}
	virtual void sim_trigger(const trigger_type &)
		{}

	// These members keep track of where the delay or event control was used.
	bool _persist; // Persist or delete after use.
	bool _delayed_loop; // Delayed while in a loop.
	};

inline ostream &
operator<<(ostream &s, const dec_base &d)
	{ return d.display(s); }

struct dec_setup
	{
	virtual void operator()(delay_num *) const {}
	virtual void operator()(delay_id *) const {}
	virtual void operator()(ored_event_expr *) const {}
	};

struct dec_trigger
	{
	virtual void operator()(delay_num *) const {}
	virtual void operator()(delay_id *) const {}
	virtual void operator()(ored_event_expr *) const {}
	};

#endif // _DECBASE_H
