// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// decsetup.cc

#include "common/error.h"
#include "common/sym_tab.h"
#include "common/st_net.h"
#include "expr/esetup.h"
#include "expr/eeval.h"
#include "expr/emon.h"
#include "misc/delayid.h"
#include "misc/evntexpr.h"
#include "misc/decsetup.h"

extern symbol_table symboltable;


setup_event_expr::setup_event_expr(scopelist_type &s)
	: _scope(s)
	{}

void
setup_event_expr::operator()(event_expr *p) const
	{
	if (p->_type != TRIG_BITCHNG
	 && p->_type != TRIG_POSEDGE
	 && p->_type != TRIG_NEGEDGE)
		{
		strstream buf;
		buf << *p;
		sim_seterror(SE_INTERNAL, p->_filename, p->_lineno);
		sim_perror(buf);
		}

	// Setup expression for later use.
	p->_expr->sim_setup(setup_expr(_scope));

	// Check to make sure we can actually monitor this expression.
	if (!p->_expr->sim_monitor(monitor_expr(0)))
		{
		strstream buf;
		buf << *(p->_expr);
		sim_seterror(SE_TYPE, p->_filename, p->_lineno);
		sim_perror(buf);
		}

	// If the monitor type is a level change, then the expression
	// must evaluate to a single bit.
	if (p->_type == TRIG_BITCHNG)
		{
		num_type result(p->_expr->sim_evaluate(evaluate_expr()));
		if (result.size() != 1)
			{
			strstream buf;
			buf << *(p->_expr);
			sim_seterror(SE_TYPE, p->_filename, p->_lineno);
			sim_perror(buf);
			}
		}
	}


// Setup delay or event control function object.

setup_dec::setup_dec(scopelist_type &s)
	: _scope(s)
	{}

void
setup_dec::operator()(delay_id *p) const
	{
	// Check range id.
	p->_index = symboltable.find(p->_name, _scope);
	if (p->_index._scope < 0)
		{
		sim_seterror(SE_STDEF, p->_filename, p->_lineno);
		sim_perror(p->_name);
		}
	// I don't think this should be a net, but we have no other valid
	// node type.
	net_type *node = symboltable.get(p->_index)->get_net();
	if (node == 0)
		{
		sim_seterror(SE_STDEF, p->_filename, p->_lineno);
		sim_perror(p->_name);
		}
	}

void
setup_dec::operator()(ored_event_expr *p) const
	{
	ored_event_expr::event_expr_list::iterator itp(p->_ored_list->begin());
	ored_event_expr::event_expr_list::iterator stop(p->_ored_list->end());
	for (; itp != stop; ++itp)
		(*itp).get()->sim_setup(setup_event_expr(_scope));
	}
