// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// evntexpr.h
//
// Event expression:  this class and OredEvntExpression must be defined
// because Sim_setup of EvntExpression requires instantiating Event<Stmts>.
// Thus, Stmts must be defined before this class.

#ifndef _EVNTEXPR_H
#define _EVNTEXPR_H

#include <list>
#include "common/ptr.h"
#include "common/combase.h"
#include "common/event.h" // TRIG_NONE.
#include "expr/exprbase.h"
#include "misc/decbase.h"

struct event_expr_setup;
struct event_expr_trigger;

class event_expr : public virtual common_base
	{
	friend struct trigger_event_expr;
	friend struct setup_event_expr;

public:
	typedef expr_base expr_type;

	typedef event_expr_setup setup_type;
	typedef event_expr_trigger trigger_type;

	event_expr(expr_type *, TRIG_TYPE = TRIG_NONE);
	event_expr(const event_expr &);
	~event_expr();

	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	void sim_trigger(const trigger_type &);

private:
	TRIG_TYPE _type;
	expr_type *_expr;
	};

class ored_event_expr : public dec_base
	{
	friend struct trigger_dec;
	friend struct setup_dec;

public:
	typedef event_expr event_expr_type;
	typedef basic_ptr<event_expr_type> event_expr_ptr;
	typedef list<event_expr_ptr> event_expr_list;

	ored_event_expr(event_expr_list *);
	ored_event_expr(const ored_event_expr &);
	~ored_event_expr();

	ored_event_expr *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	void sim_trigger(const trigger_type &);

private:
	event_expr_list *_ored_list;
	};

ostream &operator<<(ostream &, const event_expr &);
ostream &operator<<(ostream &, const ored_event_expr &);

struct event_expr_setup
	{
	virtual void operator()(event_expr *) const {}
	};

struct event_expr_trigger
	{
	virtual void operator()(event_expr *) const {}
	};

#endif // _EVNTEXPR_H
