// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// msetup.cc

#include "common/hash.h"
#include "common/error.h"
#include "common/sym_tab.h"
#include "common/st_mod.h"
#include "common/st_net.h"
#include "expr/rangeid.h"
#include "expr/esetup.h"
#include "misc/msetup.h"

extern symbol_table symboltable;

// sim_parsed_module

setup_module::setup_module(module_type *p)
	{ _stmod = p; }

void
setup_module::operator()(module *p) const
	{
	// Setting up a module means storing the port list and module items.
	// This is all we do because we are not really in setup mode yet.
	_stmod->_port_list = p->_port_list;
	_stmod->_module_item = p->_module_item;
	// Steal the lists.
	p->_port_list = 0;
	p->_module_item = 0;
	}


setup_port::setup_port(scopelist_type &s, port_conn_type *p)
	: _scope(s) { _port_conn = p; }

void
setup_port::operator()(port *p) const
	{
	// Setting up a port simply requires us to create a net type symbol
	// table object.
	net_type *net = new net_type(p->_name);
	net->_scope = _scope.top();
	net_type::hash_type hv(symboltable.add(net));
	if (hv._scope < 0)
		{
		delete net;
		sim_seterror((SIM_ERR_NUM) hv._value, p->_filename, p->_lineno);
		sim_perror(p->_name);
		}

	// All ports need a port connection.  Setup the connection...
	ident_type *id = _port_conn->_expr->get_range_id();

	// Port index points to the index of the port connected to.
	net->_port_index = new net_type::hash_type(id->index());
	id->get_select(net->_port_ms, net->_port_ls);

	// Now add ourself to the propagate list of the port connection.
	net_type *conn = symboltable.get(id->index())->get_net();
	if (conn == 0)
		{
		sim_seterror(SE_TYPE, id->_filename, id->_lineno);
		sim_perror(id->name());
		}
	if (!conn->_propagate)
		conn->_propagate = new net_type::hash_list;
	conn->_propagate->push_back(hv);
	}


setup_port_conn::setup_port_conn(scopelist_type &s)
	: _scope(s)
	{}

void
setup_port_conn::operator()(port_connection *p) const
	{
	ident_type *id = p->_expr->get_range_id();
	if (id == 0)
		{
		strstream buf;
		buf << p->_expr;
		sim_seterror(SE_TYPE, p->_filename, p->_lineno);
		sim_perror(buf);
		}
	// We need to setup the range id.  Since port setup is done during
	// module instantiation, and module instantiations are usually done
	// before anything else.  The range id might not be setup yet.
	id->sim_setup(setup_expr(_scope));
	}


setup_select::setup_select(scopelist_type &s)
	: _scope(s)
	{}

void
setup_select::operator()(bit *p) const
	{ p->_expr->sim_setup(setup_expr(_scope)); }
