// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// d_in.cc

#include "expr/rangeid.h"
#include "moditm/d_in.h"

input_decl::input_decl(ident_list *i, range_type *r)
	{ _range = r; _id_list = i; }

input_decl::~input_decl()
	{ delete _range; delete _id_list; }

input_decl::ostream_type &
input_decl::display(ostream_type &s) const
	{
	s << "input ";
	if (_range != 0)
		s << *_range;
	if (_id_list != 0)
		{
		ident_list::iterator itp(_id_list->begin());
		ident_list::iterator stop(_id_list->end());
		(*itp)->display(s);
		for (++itp; itp != stop; ++itp)
			{
			s << ", ";
			(*itp)->display(s);
			}
		}
	s << ';' << endl;
	return s;
	}

void
input_decl::sim_setup(const setup_type &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const input_decl &d)
	{ return d.display(s); }
