// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// modinst.cc

#include "expr/exprbase.h" // port connection needs to instantiate expression.
#include "misc/portconn.h"
#include "moditm/modinst.h"

module_instance::module_instance(const char *n, port_conn_list *c)
	: _name(n) { _port_conn = c; }

module_instance::~module_instance()
	{ delete _port_conn; }

module_instance::ostream_type &
module_instance::display(ostream_type &s) const
	{
	s << _name << " (";
	port_conn_list::iterator idx(_port_conn->begin());
	port_conn_list::iterator stop(_port_conn->end());
	s << *(*idx);
	for (++idx; idx != stop; ++idx)
		s << "," << *(*idx);
	s << ")";
	return s;
	}

void
module_instance::sim_setup(const module_instance_setup &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const module_instance &d)
	{ return d.display(s); }


module_instantiation::module_instantiation(const char *n, mod_inst_list *i)
	: _name(n) { _instance = i; }

module_instantiation::~module_instantiation()
	{ delete _instance; }

module_instantiation::ostream_type &
module_instantiation::display(ostream_type &s) const
	{
	s << _name << " ";
	mod_inst_list::iterator idx(_instance->begin());
	mod_inst_list::iterator stop(_instance->end());
	s << *(*idx);
	for (++idx; idx != stop; ++idx)
		s << "," << endl << *(*idx) << endl;
	return s;
	}

void
module_instantiation::sim_setup(const setup_type &setup)
	{ setup(this); }

ostream &
operator<<(ostream &s, const module_instantiation &d)
	{ return d.display(s); }
