// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// modinst.h
//
// This file contains all objects needed for module instantiation.
// Which are module instance, and port connection objects.

#ifndef _MODINST_H
#define _MODINST_H

#include <list>
#include "common/ptr.h"
#include "common/combase.h"
#include "misc/portconn.h"
#include "moditm/mibase.h"

struct module_instance_setup;

class module_instance : public virtual common_base
	{
	friend struct setup_module_instance;

public:
	typedef port_connection port_conn;
	typedef basic_ptr<port_conn> port_conn_ptr;
	typedef list<port_conn_ptr> port_conn_list;

	typedef module_instance_setup setup_type;;

	module_instance(const char *, port_conn_list *);
	~module_instance();

	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);

private:
	str_type _name;
	port_conn_list *_port_conn;
	};

class module_instantiation : public module_item_base
	{
	friend struct setup_module_item;

public:
	typedef module_instance mod_inst;
	typedef basic_ptr<mod_inst> mod_inst_ptr;
	typedef list<mod_inst_ptr> mod_inst_list;

	typedef mitf_setup setup_type;

	module_instantiation(const char *, mod_inst_list *);
	~module_instantiation();

	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);

private:
	str_type _name;
	mod_inst_list *_instance;
	};

struct module_instance_setup
	{
	virtual void operator()(module_instance *) const {}
	};

ostream &operator<<(ostream &, const module_instance &);
ostream &operator<<(ostream &, const module_instantiation &);

#endif // _MODINST_H
