// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// p_misc.cc

#include <stdlib.h>
#include <iostream.h>
#include <list>
#include "expr/number.h" // delay num.
#include "expr/rangeid.h" // event id.
#include "misc/module.h"
#include "misc/port.h"
#include "misc/delaynum.h"
#include "misc/delayid.h"
#include "misc/evntexpr.h"
#include "misc/cexpr.h"
#include "misc/bit.h"
#include "misc/range.h"
#include "moditm/mibase.h"
#include "parser.h"
#include "p_misc.h"

// Module objects.

p_module
p_create_module(char *name, p_moduleitemlist milst, p_portlist plst)
	{
	module *ret;
	if (plst != 0)
		ret = new module(name,
			(list< basic_ptr<module_item_base> > *)milst,
			(list< basic_ptr<port> > *)plst);
	else
		ret = new module(name,
			(list< basic_ptr<module_item_base> > *)milst);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	free(name);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created a module." << endl;
#endif
	return ret;
	}

p_portlist
p_create_portlist(p_portlist plst, p_port p)
	{
	list< basic_ptr<port> > *ret;
	if (plst == 0)
		ret = new list< basic_ptr<port> >;
	else
		ret = (list< basic_ptr<port> > *)plst;
	if (p != 0)
		{
		ret->push_back((port *)p);
#ifdef DEBUG_PARSER
		cout << "PARSER:  added port to list." << endl;
#endif
		}
	return ret;
	}

p_port
p_create_port(char *name)
	{
	port *ret;
	ret = new port(name);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	free(name);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created port." << endl;
#endif
	return ret;
	}


/* Delay or event control objects. */

p_delayevntctl
p_create_delaynum(p_number num)
	{
	delay_num *ret;
	bit_vector *amt = (number *)num;
	ret = new delay_num(*amt);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	delete amt;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created delay number." << endl;
#endif
	return ret;
	}

p_delayevntctl
p_create_delayid(char *id)
	{
	delay_id *ret;
	ret = new delay_id(id);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	free(id);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created delay identifier." << endl;
#endif
	return ret;
	}

p_delayevntctl
p_create_evntid(char *id)
	{
	event_expr *evntexpr;
	range_id *rid = new range_id(id, 0);
	evntexpr = (event_expr *)p_create_chngevntexpr(rid);
	list< basic_ptr<event_expr> > *lst = (list< basic_ptr<event_expr> > *)
		p_create_evntexprlist(0, evntexpr);
	ored_event_expr *ret = (ored_event_expr *)p_create_orevntexpr(lst);
	free(id);
	return ret;
	}

p_delayevntctl
p_create_orevntexpr(p_evntexprlist lst)
	{
	ored_event_expr *ret;
	ret = new ored_event_expr((list< basic_ptr<event_expr> > *)lst);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created ored event expression." << endl;
#endif
	return ret;
	}

p_evntexpr
p_create_chngevntexpr(p_expression exp)
	{
	event_expr *ret;
	ret = new event_expr((expr_base *)exp, TRIG_BITCHNG);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created event expression (change)." << endl;
#endif
	return ret;
	}

p_evntexpr
p_create_posevntexpr(p_expression exp)
	{
	event_expr *ret;
	ret = new event_expr((expr_base *)exp, TRIG_POSEDGE);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created event expression (posedge)." << endl;
#endif
	return ret;
	}

p_evntexpr
p_create_negevntexpr(p_expression exp)
	{
	event_expr *ret;
	ret = new event_expr((expr_base *)exp, TRIG_NEGEDGE);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created event expression (negedge)." << endl;
#endif
	return ret;
	}

p_evntexprlist
p_create_evntexprlist(p_evntexprlist lst, p_evntexpr exp)
	{
	list< basic_ptr<event_expr> > *ret;
	if (lst != 0)
		ret = (list< basic_ptr<event_expr> > *)lst;
	else
		ret = new list< basic_ptr<event_expr> >;
	ret->push_back((event_expr *)exp);
#ifdef DEBUG_PARSER
	cout << "PARSER:  added event expression to list." << endl;
#endif
	return ret;
	}

p_constexpr
p_create_constexpr(p_expression exp)
	{
	const_expr *ret;
	ret = new const_expr((expr_base *) exp);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	delete (expr_base *) exp;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created const expr." << endl;
#endif
	return ret;
	}

p_bit
p_create_bit(p_expression exp)
	{
	bit *ret;
	ret = new bit((expr_base *) exp);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created bit select." << endl;
#endif
	return ret;
	}

p_range
p_create_range(p_constexpr l, p_constexpr r)
	{
	range *ret;
	ret = new range((const_expr *) l, (const_expr *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
	delete (const_expr *) l;
	delete (const_expr *) r;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created range select." << endl;
#endif
	return ret;
	}
