// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// p_moditm.cc

#include "stmt/stmtbase.h"
#include "moditm/initial.h"
#include "moditm/always.h"
#include "moditm/func.h"
#include "moditm/task.h"
#include "moditm/modinst.h"
#include "moditm/cassign.h"
#include "moditm/d_reg.h"
#include "moditm/d_wire.h"
#include "moditm/d_in.h"
#include "moditm/d_out.h"
#include "moditm/d_inout.h"
#include "parser.h"
#include "p_moditm.h"

// Module item objects.

p_moduleitemlist
p_create_moduleitemlist(p_moduleitemlist milst, p_moduleitem mi)
	{
	list< basic_ptr<module_item_base> > *ret;
	if (milst == 0)
		ret = new list< basic_ptr<module_item_base> >;
	else
		ret = (list< basic_ptr<module_item_base> > *)milst;
	if (mi != 0)
		{
		ret->push_back((module_item_base *)mi);
#ifdef DEBUG_PARSER
		cout << "PARSER:  added module item to list." << endl;
#endif
		}
	return ret;
	}

p_initial
p_create_initial(p_stmts st)
	{
	initial *ret;
	ret = new initial((stmt_base *)st);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created initial statement." << endl;
#endif
	return ret;
	}

p_always
p_create_always(p_stmts st)
	{
	always *ret;
	ret = new always((stmt_base *)st);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created always statement." << endl;
#endif
	return ret;
	}

p_regdecl
p_create_regdecl(p_rangeidlist idlst, p_range r)
	{
	reg_decl *ret;
	ret = new reg_decl((list< basic_ptr<range_id> > *) idlst, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created register declaration." << endl;
#endif
	return ret;
	}

p_wiredecl
p_create_wiredecl(p_rangeidlist idlst, p_range r)
	{
	wire_decl *ret;
	ret = new wire_decl((list< basic_ptr<range_id> > *) idlst, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created wire declaration." << endl;
#endif
	return ret;
	}

p_inputdecl
p_create_inputdecl(p_rangeidlist idlst, p_range r)
	{
	input_decl *ret;
	ret = new input_decl((list< basic_ptr<range_id> > *) idlst, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created input declaration." << endl;
#endif
	return ret;
	}

p_outputdecl
p_create_outputdecl(p_rangeidlist idlst, p_range r)
	{
	output_decl *ret;
	ret = new output_decl((list< basic_ptr<range_id> > *) idlst, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created output declaration." << endl;
#endif
	return ret;
	}

p_inoutdecl
p_create_inoutdecl(p_rangeidlist idlst, p_range r)
	{
	inout_decl *ret;
	ret = new inout_decl((list< basic_ptr<range_id> > *) idlst, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created inout declaration." << endl;
#endif
	return ret;
	}

p_tfdecllist
p_create_tfdecllist(p_tfdecllist tflst, p_tfdecl tf)
	{
	list< basic_ptr<tfdecl_base> > *ret;
	if (tflst == 0)
		ret = new list< basic_ptr<tfdecl_base> >;
	else
		ret = (list< basic_ptr<tfdecl_base> > *)tflst;
	if (tf != 0)
		{
		ret->push_back((tfdecl_base *)tf);
#ifdef DEBUG_PARSER
		cout << "PARSER:  added tf declaration to list." << endl;
#endif
		}
	return ret;
	}

p_function
p_create_function(const char *name, p_stmts st, p_tfdecllist tflist, p_range r)
	{
	function *ret;
	ret = new function(name, (stmt_base *) st,
		(list< basic_ptr<tfdecl_base> > *) tflist, (range *) r);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created function definition." << endl;
#endif
	return ret;
	}

p_task
p_create_task(const char *name, p_stmts st, p_tfdecllist tflist)
	{
	task *ret;
	ret = new task(name, (stmt_base *) st,
		(list< basic_ptr<tfdecl_base> > *) tflist);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created task definition." << endl;
#endif
	return ret;
	}

p_portconn
p_create_portconn(p_expression exp)
	{
	port_connection *ret;
	ret = new port_connection((expr_base *) exp);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created port connection." << endl;
#endif
	return ret;
	}

p_portconnlist
p_create_portconnlist(p_portconnlist lst, p_portconn pc)
	{
	list< basic_ptr<port_connection> > *ret;
	if (lst != 0)
		ret = (list< basic_ptr<port_connection> > *) lst;
	else
		ret = new list< basic_ptr<port_connection> >;
	ret->push_back((port_connection *)pc);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created port connection list." << endl;
#endif
	return ret;
	}

p_modinstance
p_create_modinstance(const char *name, p_portconnlist lst)
	{
	module_instance *ret;
	ret = new module_instance(name, (list< basic_ptr<port_connection> > *)lst);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created module instance." << endl;
#endif
	return ret;
	}

p_modinstancelist
p_create_modinstancelist(p_modinstancelist lst, p_modinstance mi)
	{
	list< basic_ptr<module_instance> > *ret;
	if (lst != 0)
		ret = (list< basic_ptr<module_instance> > *) lst;
	else
		ret = new list< basic_ptr<module_instance> >;
	ret->push_back((module_instance *)mi);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created module instance list." << endl;
#endif
	return ret;
	}

p_modinstan
p_create_modinstan(const char *name, p_modinstancelist lst)
	{
	module_instantiation *ret;
	ret = new module_instantiation(name,
		(list< basic_ptr<module_instance> > *)lst);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created module instantiation." << endl;
#endif
	return ret;
	}

p_blkassignlist
p_create_blkassignlist(p_blkassignlist lst, p_blkassignstmt as)
	{
	list< basic_ptr<block_assign_stmt> > *ret;
	if (lst != 0)
		ret = (list< basic_ptr<block_assign_stmt> > *) lst;
	else
		ret = new list< basic_ptr<block_assign_stmt> >;
	ret->push_back((block_assign_stmt *)as);
#ifdef DEBUG_PARSER
	cout << "PARSER:  created assignment list." << endl;
#endif
	return ret;
	}

p_contassign
p_create_contassign(p_blkassignlist lst, p_delayevntctl d)
	{
	cont_assign *ret;
	ret = new cont_assign((list< basic_ptr<block_assign_stmt> > *)lst,
		(dec_base *)d);
	ret->_filename = cur_filename;
	ret->_lineno = cur_lineno;
#ifdef DEBUG_PARSER
	cout << "PARSER:  created continuous assignment." << endl;
#endif
	return ret;
	}
