/*
 * Verilog Behavioral Simulator
 * Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
 *
 * This file is part of the Verilog Behavioral Simulator package.
 * See the file COPYRIGHT for copyright and disclaimer information.
 * See the file COPYING for the licensing terms and conditions.
 * See the file CONTRIBUTORS for a list of contributing authors.
 *
 * p_types.h
 *
 * This file contains all the typedef's used by the p_*.h header files.
 * The reason why they are here is because the objects became complex
 * enough to be interdependent on each other.  Thus, we need to typedef
 * all the types in one location.
 */

#ifndef _P_TYPES_H
#define _P_TYPES_H

#ifdef __cplusplus
extern "C" {
#endif

/* Types used in expression objects. */

typedef void *p_select;
typedef void *p_bit;
typedef void *p_range;

typedef void *p_expression;
typedef void *p_expressionlist;
typedef void *p_constexpr;
typedef void *p_rangeid;
typedef void *p_rangeidlist;
typedef void *p_qstring;
typedef void *p_number;

typedef void *p_add;
typedef void *p_subtract;

typedef void *p_invert;
typedef void *p_logand;
typedef void *p_logor;
typedef void *p_logxor;
typedef void *p_lshift;
typedef void *p_rshift;

typedef void *p_not;
typedef void *p_eqeq;
typedef void *p_noteq;
typedef void *p_lesthan;
typedef void *p_leseq;
typedef void *p_grtthan;
typedef void *p_grteq;

typedef void *p_oror;
typedef void *p_andand;

typedef void *p_funcenable;


/* Types used in statement objects. */

typedef void *p_lvalue;
typedef void *p_caseitem;
typedef void *p_caseitemlist;

typedef void *p_stmts;
typedef void *p_stmtslist;
typedef void *p_sequentialblock;
typedef void *p_taskstmt;
typedef void *p_blkassignstmt;
typedef void *p_nblkassignstmt;
typedef void *p_ifstmt;
typedef void *p_ifelsestmt;
typedef void *p_forstmt;
typedef void *p_foreverstmt;
typedef void *p_repeatstmt;
typedef void *p_whilestmt;
typedef void *p_casestmt;


/* Types used in module item objects. */

typedef void *p_moduleitemlist;
typedef void *p_moduleitem;
typedef void *p_tfdecllist;
typedef void *p_tfdecl;
typedef void *p_initial;
typedef void *p_always;
typedef void *p_regdecl;
typedef void *p_wiredecl;
typedef void *p_inputdecl;
typedef void *p_outputdecl;
typedef void *p_inoutdecl;
typedef void *p_function;
typedef void *p_task;
typedef void *p_modinstan;
typedef void *p_contassign;


/* Types used with module instantiation. */

typedef void *p_portconn;
typedef void *p_portconnlist;
typedef void *p_modinstance;
typedef void *p_modinstancelist;
typedef void *p_blkassignlist;


/* Types used in misc. objects. */

typedef void *p_module;
typedef void *p_portlist;
typedef void *p_port;

typedef void *p_evntexpr;
typedef void *p_evntexprlist;
typedef void *p_delayevntctl;


#ifdef __cplusplus
}
#endif

#endif /* _P_TYPES_H */
