/*
 * Verilog Behavioral Simulator
 * Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
 *
 * This file is part of the Verilog Behavioral Simulator package.
 * See the file COPYRIGHT for copyright and disclaimer information.
 * See the file COPYING for the licensing terms and conditions.
 * See the file CONTRIBUTORS for a list of contributing authors.
 *
 * p_types.c
 */

#include <stdio.h> /* vsprintf */
#include <string.h>
#include <stdlib.h> /* exit */
#include <ctype.h>
#include <stdarg.h> /* varargs */
#include "parser.h"

static char **filelist; /* List of files to parse. */
static int maxfiles, nfiles; /* Number of files left to parse. */

int
yywrap(void)
	{
	/*
	 * yywarp used by yacc after EOF is returned from yyin.
	 * Let's try our hand at multiple input files.
	 */
	char *curfile = filelist[maxfiles - nfiles];

#if defined(DEBUG_PARSER)
	printf("\nFinished parsing a file...\n");
#endif

	if (nfiles > 0)
		{
		/*
		 * Close old file pointer and open new file, if any.
		 * Since <nfiles> starts at non-zero, and descreases
		 * to zero, we are actually going backwards.
		 */
		fclose(yyin);
		yyin = fopen(curfile, "r");
		if (!yyin)
			{
			printf("%s -- unable to open file.\n",
				curfile);
			exit(1);
			}
		printf("Compiling file '%s'...\n", curfile);
		cur_filename = curfile;
		cur_lineno = 1;
		nfiles--;
		return 0; /* More to do. */
		}

#if defined(DEBUG_PARSER)
	printf("\nDone parsing...\n");
#endif

	return 1; /* Nothing left. */
	}

int
p_init(int amt, char **lst)
	{
	/*
	 * Initialize parser variables.  these are global because all
	 * other files are in C++.
	 */

	filelist = lst;
	maxfiles = nfiles = amt;

	/* Setup yacc input file. */
	if (nfiles <= 0)
		return 0;

	yyin = fopen(filelist[maxfiles - nfiles], "r");
	if (!yyin)
		{
		printf("%s -- unable to open file.\n",
			filelist[maxfiles - nfiles]);
		return 0;
		}
	printf("Compiling file '%s'...\n", filelist[maxfiles - nfiles]);
	cur_filename = filelist[maxfiles - nfiles];
	nfiles--;
	return 1;
	}

int
p_start(void)
	{ return yyparse(); }

char *
my_strdup(const char *str)
	{
	char *p;
	int len;
	len = strlen(str);
	p = (char *)malloc(len + 1);
	strcpy(p, str);
	return p;
	}
