// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// bassign.cc

#include "stmt/bassign.h"

block_assign_stmt::block_assign_stmt(lvalue_type *l, expr_type *e)
	{ _lval = l; _rval = e; }

block_assign_stmt::block_assign_stmt(const block_assign_stmt &p)
	: common_base(p), stmt_base(p)
	{
	_lval = new lvalue(*p._lval);
	_rval = p._rval->copy_constructor();
	}

block_assign_stmt::~block_assign_stmt()
	{ delete _lval; delete _rval; }

void
block_assign_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	}

block_assign_stmt *
block_assign_stmt::copy_constructor() const
	{ return new block_assign_stmt(*this); }

block_assign_stmt::ostream_type &
block_assign_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << *_lval << " = " << *_rval << ';';
	return s;
	}

void
block_assign_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
block_assign_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const block_assign_stmt &d)
	{ return d.display(s); }
