// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// case.h

#ifndef _CASE_H
#define _CASE_H

#include <list>
#include "common/ptr.h"
#include "common/combase.h"
#include "expr/exprbase.h"
#include "stmt/stmtbase.h"

struct case_item_setup;
struct case_item_trigger;

class case_item : public virtual common_base
	{
	friend struct setup_case_item;
	friend struct trigger_case_item;

public:
	typedef expr_base expr_type;
	typedef basic_ptr<expr_type> expr_ptr;
	typedef list<expr_ptr> expr_list;
	typedef stmt_base stmt_type;

	typedef case_item_setup setup_type;
	typedef case_item_trigger trigger_type;

	case_item(stmt_type *, expr_list * = 0);
	case_item(const case_item &);
	~case_item();

	bool is_default() const
		{ return (_expr == 0); }
	void keep_dec(bool);
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	stmt_type *_stmt;
	expr_list *_expr;
	};

class case_stmt : public stmt_base
	{
	friend struct setup_stmt;
	friend struct trigger_stmt;

public:
	typedef expr_base expr_type;
	typedef case_item citem_type;
	typedef basic_ptr<citem_type> citem_ptr;
	typedef list<citem_ptr> citem_list;

	case_stmt(expr_type *, citem_list *);
	case_stmt(const case_stmt &);
	~case_stmt();

	void keep_dec(bool);
	case_stmt *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	citem_ptr _def; // Default case, set by setup.
	expr_type *_expr;
	citem_list *_case_item;
	};

ostream &operator<<(ostream &, const case_item &);
ostream &operator<<(ostream &, const case_stmt &);

struct case_item_setup
	{
	virtual void operator()(case_item *) const {}
	};

struct case_item_trigger
	{
	virtual bool operator()(case_item *) const { return true; }
	};

#endif // _CASE_H
