// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// for.cc

#include "expr/exprbase.h"
#include "stmt/bassign.h"
#include "stmt/for.h"

for_stmt::for_stmt(block_assign_stmt *pre, expr_type *e,
		block_assign_stmt *post, stmt_type *s)
	: _delayed(false)
	{
	_expr = e;
	_stmt = s;
	_preassign = pre;
	_postassign = post;
	}

for_stmt::for_stmt(const for_stmt &p)
	: common_base(p), stmt_base(p), _delayed(p._delayed)
	{
	_expr = p._expr->copy_constructor();
	_stmt = p._stmt->copy_constructor();
	_preassign = p._preassign->copy_constructor();
	_postassign = p._postassign->copy_constructor();
	}

for_stmt::~for_stmt()
	{
	delete _expr;
	delete _stmt;
	delete _preassign;
	delete _postassign;
	}

void
for_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_stmt->keep_dec(b);
	}

for_stmt *
for_stmt::copy_constructor() const
	{ return new for_stmt(*this); }

for_stmt::ostream_type &
for_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "for (" << *_preassign << "; ";
	s << *_expr << "; ";
	s << *_postassign << ")" << endl;
	s << *_stmt;
	return s;
	}

void
for_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
for_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const for_stmt &d)
	{ return d.display(s); }
