// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// for.h

#ifndef _FOR_H
#define _FOR_H

#include "expr/exprbase.h"
#include "stmt/stmtbase.h"

class for_stmt : public stmt_base
	{
	friend struct setup_stmt;
	friend struct trigger_stmt;

public:
	typedef expr_base expr_type;
	typedef stmt_base stmt_type;
	typedef block_assign_stmt bassign_type;

	for_stmt(block_assign_stmt *, expr_type *,
		block_assign_stmt *, stmt_type *);
	for_stmt(const for_stmt &);
	~for_stmt();

	void keep_dec(bool);
	for_stmt *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	bool _delayed; // For delays in loops.
	expr_type *_expr;
	stmt_type *_stmt;
	bassign_type *_preassign;
	bassign_type *_postassign;
	};

ostream &operator<<(ostream &, const for_stmt &);

#endif // _FOR_H
