// Verilog Behavioral Simulator
// Copyright (C) 1996 Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// Contributed by Udi Finkelstein, modified by Jimen Ching.
//
// forever.cc

#include "stmt/forever.h"

forever_stmt::forever_stmt(stmt_type *s)
	: _delayed(false)
	{ _stmt = s; }

forever_stmt::forever_stmt(const forever_stmt &p)
	: common_base(p), stmt_base(p), _delayed(p._delayed)
	{ _stmt = p._stmt->copy_constructor(); }

forever_stmt::~forever_stmt()
	{ delete _stmt; }

void
forever_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_stmt->keep_dec(b);
	}

forever_stmt *
forever_stmt::copy_constructor() const
	{ return new forever_stmt(*this); }

forever_stmt::ostream_type &
forever_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "forever" << endl;
	s << *_stmt << endl;
	return s;
	}

void
forever_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
forever_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const forever_stmt &d)
	{ return d.display(s); }
