// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// if.cc

#include "expr/exprbase.h"
#include "stmt/if.h"

if_stmt::if_stmt(expr_type *e, stmt_type *s)
	{ _expr = e; _stmt = s; }

if_stmt::if_stmt(const if_stmt &p)
	: common_base(p), stmt_base(p)
	{
	_expr = p._expr->copy_constructor();
	_stmt = p._stmt->copy_constructor();
	}

if_stmt::~if_stmt()
	{ delete _expr; delete _stmt; }

void
if_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_stmt->keep_dec(b);
	}

if_stmt *
if_stmt::copy_constructor() const
	{ return new if_stmt(*this); }

if_stmt::ostream_type &
if_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "if (" << *_expr << ")" << endl;
	s << *_stmt;
	return s;
	}

void
if_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
if_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const if_stmt &d)
	{ return d.display(s); }
