// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// ifelse.cc

#include "expr/exprbase.h"
#include "stmt/ifelse.h"

if_else_stmt::if_else_stmt(expr_type *e, stmt_type *t, stmt_type *f)
	{ _expr = e; _true_stmt = t; _false_stmt = f; }

if_else_stmt::if_else_stmt(const if_else_stmt &p)
	: common_base(p), stmt_base(p)
	{
	_expr = p._expr->copy_constructor();
	_true_stmt = p._true_stmt->copy_constructor();
	_false_stmt = p._false_stmt->copy_constructor();
	}

if_else_stmt::~if_else_stmt()
	{ delete _expr; delete _true_stmt; delete _false_stmt; }

void
if_else_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_true_stmt->keep_dec(b);
	_false_stmt->keep_dec(b);
	}

if_else_stmt *
if_else_stmt::copy_constructor() const
	{ return new if_else_stmt(*this); }

if_else_stmt::ostream_type &
if_else_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "if (" << *_expr << ")" << endl;
	s << *_true_stmt << endl;
	s << "else" << endl;
	s << *_false_stmt;
	return s;
	}

void
if_else_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
if_else_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const if_else_stmt &d)
	{ return d.display(s); }
