// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// nbassign.cc

#include "stmt/nbassign.h"

non_block_assign_stmt::non_block_assign_stmt(lvalue_type *l, expr_type *e)
	{
	_value = 0;
	event_cache_type *c = new event_cache_type(this);
	_event = new change_event<stmt_type>(c, DC);
	_lval = l;
	_rval = e;
	}

non_block_assign_stmt::non_block_assign_stmt(const non_block_assign_stmt &p)
	: common_base(p), stmt_base(p)
	{
	_value = 0;
	event_cache_type *c = new event_cache_type(this);
	_event = new change_event<stmt_type>(c, DC);
	_lval = new lvalue(*p._lval);
	_rval = p._rval->copy_constructor();
	}

non_block_assign_stmt::~non_block_assign_stmt()
	{
	delete _value;
	delete _event;
	delete _lval;
	delete _rval;
	}

void
non_block_assign_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	}

non_block_assign_stmt *
non_block_assign_stmt::copy_constructor() const
	{ return new non_block_assign_stmt(*this); }

non_block_assign_stmt::ostream_type &
non_block_assign_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << *_lval << " <= " << *_rval << ';';
	return s;
	}

void
non_block_assign_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
non_block_assign_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const non_block_assign_stmt &d)
	{ return d.display(s); }
