// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// nbassign.h

#ifndef _NBASSIGN_H
#define _NBASSIGN_H

#include "common/event.h"
#include "expr/exprbase.h"
#include "stmt/stmtbase.h"
#include "stmt/lvalue.h"

class non_block_assign_stmt : public stmt_base
	{
	friend struct setup_stmt;
	friend struct trigger_stmt;

public:
	typedef lvalue lvalue_type;
	typedef expr_base expr_type;
	typedef expr_type::num_type num_type;
	typedef stmt_base stmt_type;
	typedef event_base<stmt_type> event_type;
	typedef event_type::event_container event_cache_type;

	non_block_assign_stmt(lvalue_type *, expr_type *);
	non_block_assign_stmt(const non_block_assign_stmt &);
	~non_block_assign_stmt();

	void keep_dec(bool);
	non_block_assign_stmt *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	num_type *_value;
	event_type *_event;
	lvalue_type *_lval;
	expr_type *_rval;
	};

ostream &operator<<(ostream &s, const non_block_assign_stmt &);

#endif // _NBASSIGN_H
