// Verilog Behavioral Simulator
// Copyright (C) 1996 Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// Contributed by Udi Finkelstein, modified by Jimen Ching.
//
// repeat.cc

#include "expr/exprbase.h"
#include "stmt/repeat.h"

repeat_stmt::repeat_stmt(expr_type *e, stmt_type *s)
	: _delayed(false), _repeat_count(0)
	{ _expr = e; _stmt = s; }

repeat_stmt::repeat_stmt(const repeat_stmt &p)
	: common_base(p), stmt_base(p),
		_delayed(p._delayed), _repeat_count(p._repeat_count)
	{
	_expr = p._expr->copy_constructor();
	_stmt = p._stmt->copy_constructor();
	}

repeat_stmt::~repeat_stmt()
	{ delete _expr; delete _stmt; }

void
repeat_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_stmt->keep_dec(b);
	}

repeat_stmt *
repeat_stmt::copy_constructor() const
	{ return new repeat_stmt(*this); }

repeat_stmt::ostream_type &
repeat_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "repeat (" << *_expr << ')' << endl;
	s << *_stmt << endl;
	return s;
	}

void
repeat_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
repeat_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const repeat_stmt &d)
	{ return d.display(s); }
