// Verilog Behavioral Simulator
// Copyright (C) 1995 Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// Contributed by Udi Finkelstein, modified by Jimen Ching.
//
// repeat.h

#ifndef _REPEAT_H
#define _REPEAT_H

#include "stmt/stmtbase.h"

struct expr_base;

class repeat_stmt : public stmt_base
	{
	friend struct setup_stmt;
	friend struct trigger_stmt;

public:
	typedef expr_base expr_type;
	typedef stmt_base stmt_type;

	repeat_stmt(expr_type *, stmt_type *);
	repeat_stmt(const repeat_stmt &);
	~repeat_stmt();

	void keep_dec(bool);
	repeat_stmt *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	bool _delayed; // For delays in loops.
	unsigned long _repeat_count; // For triggering statement.
	expr_type *_expr;
	stmt_type *_stmt;
	};

ostream &operator<<(ostream &, const repeat_stmt &);

#endif // _REPEAT_H
