// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// ssetup.h

#ifndef _SSETUP_H
#define _SSETUP_H

#include "common/dstack.h"
#include "common/event.h"
#include "stmt/stmtbase.h"
#include "stmt/lvalue.h"
#include "stmt/case.h"

struct st_node_base;
class st_function;
class st_task;
class st_net;

struct setup_lvalue : public lvalue_setup
	{
	typedef Stack<int> scopelist_type;
	typedef st_node_base node_type;

	setup_lvalue(scopelist_type &);
	void operator()(lvalue *) const;

	scopelist_type &_scope;
	};

struct setup_case_item : public case_item_setup
	{
	typedef Stack<int> scopelist_type;

	setup_case_item(scopelist_type &);
	void operator()(case_item *) const;

	scopelist_type &_scope;
	};

struct setup_stmt : public stmt_setup
	{
	typedef Stack<int> scopelist_type;
	typedef stmt_base stmt_type;
	typedef event_base<stmt_type> event_type;
	typedef counted_ptr<event_type> event_ptr;
	typedef event_type::event_container event_cache_type;
	typedef st_function func_type;
	typedef st_task task_type;
	typedef st_net net_type;

	setup_stmt(scopelist_type &, event_ptr * = 0);
	void operator()(null_stmt *) const;
	void operator()(seq_block_stmt *) const;
	void operator()(systask_finish *) const;
	void operator()(systask_write *) const;
	void operator()(systask_monitor *) const;
	void operator()(sysfunc_time *) const;
	void operator()(task_enable_stmt *) const;
	void operator()(block_assign_stmt *) const;
	void operator()(non_block_assign_stmt *) const;
	void operator()(if_stmt *) const;
	void operator()(if_else_stmt *) const;
	void operator()(for_stmt *) const;
	void operator()(forever_stmt *) const;
	void operator()(repeat_stmt *) const;
	void operator()(while_stmt *) const;
	void operator()(case_stmt *) const;

	scopelist_type &_scope;
	event_ptr *_event;
	};

#endif // _SSETUP_H
