// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// stmtbase.h

#ifndef _STMTBASE_H
#define _STMTBASE_H

#include "common/combase.h"
#include "expr/exprbase.h"
#include "misc/decbase.h"

class null_stmt;
class seq_block_stmt;
class systask_finish;
class systask_write;
class systask_monitor;
class sysfunc_time;
class task_enable_stmt;
class block_assign_stmt;
class non_block_assign_stmt;
class if_stmt;
class if_else_stmt;
class case_stmt;
class for_stmt;
class forever_stmt;
class repeat_stmt;
class while_stmt;

struct stmt_setup
	{
	virtual void operator()(null_stmt *) const {}
	virtual void operator()(seq_block_stmt *) const {}
	virtual void operator()(systask_finish *) const {}
	virtual void operator()(systask_write *) const {}
	virtual void operator()(systask_monitor *) const {}
	virtual void operator()(sysfunc_time *) const {}
	virtual void operator()(task_enable_stmt *) const {}
	virtual void operator()(block_assign_stmt *) const {}
	virtual void operator()(non_block_assign_stmt *) const {}
	virtual void operator()(if_stmt *) const {}
	virtual void operator()(if_else_stmt *) const {}
	virtual void operator()(case_stmt *) const {}
	virtual void operator()(for_stmt *) const {}
	virtual void operator()(forever_stmt *) const {}
	virtual void operator()(repeat_stmt *) const {}
	virtual void operator()(while_stmt *) const {}
	};

struct stmt_trigger
	{
	virtual bool operator()(null_stmt *) const { return true; }
	virtual bool operator()(seq_block_stmt *) const { return true; }
	virtual bool operator()(systask_finish *) const { return true; }
	virtual bool operator()(systask_write *) const { return true; }
	virtual bool operator()(systask_monitor *) const { return true; }
	virtual bool operator()(sysfunc_time *) const { return true; }
	virtual bool operator()(task_enable_stmt *) const { return true; }
	virtual bool operator()(block_assign_stmt *) const { return true; }
	virtual bool operator()(non_block_assign_stmt *) const { return true; }
	virtual bool operator()(if_stmt *) const { return true; }
	virtual bool operator()(if_else_stmt *) const { return true; }
	virtual bool operator()(case_stmt *) const { return true; }
	virtual bool operator()(for_stmt *) const { return true; }
	virtual bool operator()(forever_stmt *) const { return true; }
	virtual bool operator()(repeat_stmt *) const { return true; }
	virtual bool operator()(while_stmt *) const { return true; }
	};

struct stmt_base : public virtual common_base
	{
	typedef expr_base expr_type; // Used by event objects.
	typedef dec_base dec_type;

	typedef stmt_setup setup_type;
	typedef stmt_trigger trigger_type;

	stmt_base(dec_type *d = 0)
		: _always(false), _dec(d) {}
	stmt_base(const stmt_base &s)
		: common_base(s), _always(s._always)
		{
		if (s._dec != 0)
			_dec = s._dec->copy_constructor();
		else
			_dec = 0;
		}
	virtual ~stmt_base()
		{ delete _dec; }

	// get_task_enable is used for triggering, where the argument is
	// needed.  This should be changed later to dynamic_cast when
	// compilers have better support for it.
	virtual task_enable_stmt *get_task_enable()
		{ return 0; }

	virtual stmt_base *copy_constructor() const
		{ return new stmt_base(*this); }
	virtual ostream_type &display(ostream_type &s) const
		{ return s; }
	virtual void sim_setup(const setup_type &)
		{}
	virtual bool sim_trigger(const trigger_type &)
		{ return true; }

	// Keep dec means the statement should not remove the dec when
	// triggered.  This is for loop statements.
	virtual void keep_dec(bool b)
		{ if (_dec != 0) _dec->_persist = b; }

	bool _always; // Part of an always procedural block.
	dec_type *_dec; // Optional delay or event control.
	};

class null_stmt : public stmt_base
	{
public:
	ostream &display(ostream &s) const
		{ if (_dec != 0) s << *_dec << " ;"; return s; }
	};

inline ostream &
operator<<(ostream &s, const stmt_base &d)
	{ return d.display(s); }

#endif // _STMTBASE_H
