// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// strigger.h

#ifndef _STRIGGER_H
#define _STRIGGER_H

#include "expr/exprbase.h"
#include "stmt/stmtbase.h"
#include "stmt/lvalue.h"
#include "stmt/case.h"

struct st_node_base;
class st_function;
class st_task;

struct trigger_lvalue : public lvalue_trigger
	{
	typedef select_base::SELECT_SIZE select_size_type;
	typedef select_base::position_type position_type;
	typedef expr_base expr_type;
	typedef expr_type::num_type num_type;
	typedef st_node_base node_type;
	typedef st_function func_type;
	typedef st_net net_type;

	trigger_lvalue(num_type &);
	void operator()(lvalue *) const;

	num_type &_num;
	};

struct trigger_case_item : public case_item_trigger
	{
	typedef expr_base expr_type;
	typedef expr_type::num_type num_type;
	typedef stmt_base stmt_type;

	trigger_case_item(num_type &, bool &, stmt_type *);
	bool operator()(case_item *) const;

	bool &_status;
	num_type &_value;
	stmt_type *_parent;
	};

struct trigger_stmt : public stmt_trigger
	{
	typedef stmt_base stmt_type;
	typedef stmt_type::str_type str_type;
	typedef stmt_type::expr_type expr_type;
	typedef expr_type::num_type num_type;
	typedef task_enable_stmt task_caller_type;
	typedef st_function func_type;
	typedef st_task task_type;

	trigger_stmt(stmt_type *);

	bool operator()(null_stmt *) const;
	bool operator()(seq_block_stmt *) const;
	bool operator()(systask_finish *) const;
	bool operator()(systask_write *) const;
	bool operator()(systask_monitor *) const;
	bool operator()(sysfunc_time *) const;
	bool operator()(task_enable_stmt *) const;
	bool operator()(block_assign_stmt *) const;
	bool operator()(non_block_assign_stmt *) const;
	bool operator()(if_stmt *) const;
	bool operator()(if_else_stmt *) const;
	bool operator()(for_stmt *) const;
	bool operator()(forever_stmt *) const;
	bool operator()(repeat_stmt *) const;
	bool operator()(while_stmt *) const;
	bool operator()(case_stmt *) const;

	stmt_type *_parent;
	};

#endif // _STRIGGER_H
