// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// taskenbl.h

#ifndef _TASKENBL_H
#define _TASKENBL_H

#include <list>
#include "common/ptr.h"
#include "common/hash.h"
#include "expr/exprbase.h"
#include "stmt/stmtbase.h"

class task_enable_stmt : public stmt_base
	{
	friend struct setup_stmt;
	friend struct trigger_stmt;

	friend struct trigger_st_node;

public:
	typedef hash_value hash_type;
	typedef expr_base expr_type;
	typedef basic_ptr<expr_type> arg_type;
	typedef list<arg_type> arg_list;

	task_enable_stmt(const char *, arg_list * = 0);
	task_enable_stmt(const task_enable_stmt &);
	~task_enable_stmt();

	void keep_dec(bool);
	task_enable_stmt *get_task_enable();
	task_enable_stmt *copy_constructor() const;
	ostream_type &display(ostream_type &) const;
	void sim_setup(const setup_type &);
	bool sim_trigger(const trigger_type &);

private:
	str_type _name; // Name of system task w/ '$'
	hash_type _index; // Index of task in symbol table.
	arg_list *_argument; // String constant for format
                          // and list of arguments
	};

ostream &operator<<(ostream &, const task_enable_stmt &);

#endif // _TASKENBL_H
