// Verilog Behavioral Simulator
// Copyright (C) 1996 Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// Contributed by Udi Finkelstein, modified by Jimen Ching.
//
// while.cc

#include "expr/exprbase.h"
#include "stmt/while.h"

while_stmt::while_stmt(expr_type *e, stmt_type *s)
	: _delayed(false)
	{ _expr = e; _stmt = s; }

while_stmt::while_stmt(const while_stmt &p)
	: common_base(p), stmt_base(p), _delayed(p._delayed)
	{
	_expr = p._expr->copy_constructor();
	_stmt = p._stmt->copy_constructor();
	}

while_stmt::~while_stmt()
	{ delete _expr; delete _stmt; }

void
while_stmt::keep_dec(bool b)
	{
	if (_dec != 0)
		_dec->_persist = b;
	_stmt->keep_dec(b);
	}

while_stmt *
while_stmt::copy_constructor() const
	{ return new while_stmt(*this); }

while_stmt::ostream_type &
while_stmt::display(ostream_type &s) const
	{
	if (_dec != 0)
		s << *_dec << ' ';
	s << "while (" << *_expr << ')' << endl;
	s << *_stmt << endl;
	return s;
	}

void
while_stmt::sim_setup(const setup_type &setup)
	{ setup(this); }

bool
while_stmt::sim_trigger(const trigger_type &trigger)
	{ return trigger(this); }

ostream &
operator<<(ostream &s, const while_stmt &d)
	{ return d.display(s); }
