// Verilog Behavioral Simulator
// Copyright (C) 1995 Lay Hoon Tho, Jimen Ching
//
// This file is part of the Verilog Behavioral Simulator package.
// See the file COPYRIGHT for copyright and disclaimer information.
// See the file COPYING for the licensing terms and conditions.
// See the file CONTRIBUTORS for a list of contributing authors.
//
// vbs.cc
//
// Main driver.  This file is kept as simple as possible.  All work should
// be done by other modules.

#include <unistd.h> // getopt
#include <iostream.h>
#include "sim.h" // Starting point...
#include "parser.h" // Initialize parser...
#include "version.h"

void
info()
	{
	cout << "Welcome to Verilog Behavioral Simulator version ";
	cout << VERSION << '.' << endl << endl;
	cout << "Syntax:  progname [filename]" << endl;
	}

bool
sys_init(char *pn)
	{
	// Initialize system variables (globals).
	sim_init(pn);
	return true;
	}

bool
sys_debug(char *fn)
	{
	sim_debug(fn);
	return true;
	}

int
main(int argc, char *argv[])
	{
	char c;

	if (!sys_init(argv[0]))
		return 1;

	if (argc < 2)
		{
		info();
		return 1;
		}

	// Need to handle filename extensions, when we finally support
	// them.
	c = getopt(argc, argv, ":h");
	while (c != -1)
		{
		switch(c)
			{
			case 'h': case '?':
				info();
				return 0;
			}
		c = getopt(argc, argv, ":h");
		}

	if (!p_init(argc - optind, &argv[optind]))
		return 1;

	// Here we go!
	sim_start();

	return 0;
	}

